signature INIT = sig
    structure C : SQL_CLIENT

    val scratchDir : string
    val urlPrefix : string
    val boardEmail : string

    exception Access of string
    exception NeedTos

    val emailSuffix : string

    type user = {id : int, name : string, rname : string, bal : int, joined : C.timestamp,
		 app : int, shares : int}

    val rowError : string * C.value list -> 'a

    val mkUserRow : C.value list -> user

    (* Direct access to database connections *)
    val conn : unit -> C.conn

    (* Open or close a session, wrapped in a transaction *)
    val init : unit -> unit
    val done : unit -> unit

    val nextSeq : C.conn * string -> int

    (* Fun with users *)

    val lookupUser : int -> user
    val listUsers : unit -> user list
    val addUser : string * string * int * int * int -> int
    (* Pass name, real name, balance ID, and share count *)
    val modUser : user -> unit
    val deleteUser : int -> string
    val byPledge : unit -> user list

    val validUsername : string -> bool
    val userNameToId : string -> int option

    val getDb : unit -> C.conn

    val getUser : unit -> user
    val getUserId : unit -> int
    val getUserName : unit -> string

    val dateString : unit -> string

    val grandfatherUsers : unit -> unit
end
